use strict ; 
use lib "$ENV{ANTELOPE}/data/perl" ;
use Datascope ; 

our $Pf = "dbbuild" ; 
our $bad ; 

sub pfget_insist { 
    my ( $pf, $name ) = @_ ; 
    my $result ;
    eval { 
	$result = pfget($pf, $name) ; 
    } ;
    if ( $@ ) { 
	elog_log ( "failed loading parameter file $pf: $@" ) ;
	$bad++ ;
    } elsif ( ! defined $result ) { 
	elog_log ( "Can't find $name in $pf" ) ; 
	$bad++ ;
    }
    return $result ;
}

sub validate_sensor { 
    my ($pf) = @_ ; 
    $bad = 0 ; 
    $pf .= ".pf" if $pf !~ /\.pf$/ ; 
    if ( ! -f $pf ) { 
	my $SensorDir = pfget($Pf,"SensorDir" ) ; 
	if ( $SensorDir !~ m"^/" ) { 
	    $SensorDir = "$ENV{ANTELOPE}/data/$SensorDir" ; 
	}
	$pf = "$SensorDir/$pf" ;
    }
    pfget_insist($pf, "category" ) ; 
    pfget_insist($pf, "configuration" ) ; 
    pfget_insist($pf, "description" ) ; 
    pfget_insist($pf, "dfile" ) ; 
    pfget_insist($pf, "gtype" ) ; 
    pfget_insist($pf, "sensitivity" ) ; 
    pfget_insist($pf, "iunits" ) ; 
    pfget_insist($pf, "ounits" ) ; 

    my $rsptype  = pfget_insist($pf, "rsptype" ) ; 
    my $band     = pfget_insist($pf, "band" ) ; 
    my $short_period_sensor = pfget_boolean($pf, "short_period_sensor") ;
    my $sensor_type_code    = pfget($pf, "sensor_type_code") ;
    if ( $rsptype =~ /V/ ) {
        if ( $short_period_sensor && ($band =~ /b/)) {
            elog_log "mismatch in $pf: \nshort_period_sensor $short_period_sensor\nband $band\n" ;
            $bad++ ;
        }
        if ( !$short_period_sensor && ($band =~ /s/)) {
            elog_log "mismatch in $pf: \nshort_period_sensor $short_period_sensor\nband $band\n" ;
            $bad++ ;
        }
        if ( $sensor_type_code !~ /[HL]/ ) {
            elog_log "mismatch in $pf: \nrsptype $rsptype\nsensor_type_code $sensor_type_code\n" ;
            $bad++ ;
        }
    }
    if ( $rsptype =~ /A/ ) {
        if ( $short_period_sensor ) {
            elog_log "mismatch in $pf: \nrsptype $rsptype\nshort_period_sensor $short_period_sensor\n" ;
            $bad++ ;
        }
        if ( $band !~ /b/) {
            elog_log "mismatch in $pf: \nrsptype $rsptype\nband $band\n" ;
            $bad++ ;
        }
        if ( $sensor_type_code !~ /N/ ) {
            elog_log "mismatch in $pf: \nrsptype $rsptype\nsensor_type_code $sensor_type_code\n" ;
            $bad++ ;
        }
    }

    my $response = pfget_insist($pf, "response" ) ; 
    if ( $response =~ /^\*\* / ) { 
	elog_log "bad response in $pf: $response\n" ; 
	$bad++ ;
    }
    my $orient = pfget_insist($pf, "orientations" ) ; 
    my @orient = @$orient ; 
    if ( @orient < 1 ) { 
	elog_log ( "$pf doesn't have any orientations" ) ; 
	$bad++ ;
    }
    return $bad ; 
}

sub decimation_factor { 
    my ($response) = @_ ; 
    my @response = split("\n", $response) ; 
    my $flag = 0 ; 
    my ($f0, $midpoint) ;
    my $dec = 1 ; 
    foreach ( @response ) { 
	if ( /^theoretical\b/ ) { 
	    $flag = 1 ;
	} elsif ( $flag ) { 
	    ($f0, $dec, $midpoint) = split(' ', $_) ; 
	    last ; 
	}
    } 
    return $dec ; 
}

sub validate_stage { 
    my ($pf, $stagename, $rate) = @_ ; 
    my $gtype = pfget_insist($pf, "$stagename\{gtype}") ;
    my $pfname = $pf ; 
    $pfname =~ s".*/"" ; 
    if ( $gtype eq "digitizer" ) { 
	$rate = pfget_insist($pf, "$stagename\{samprate}") ;
	pfget_insist($pf, "$stagename\{iunits}") ;
	pfget_insist($pf, "$stagename\{ounits}") ;
	pfget_insist($pf, "$stagename\{sn}") ;
    } elsif ( $gtype =~ /fir/i ) { 
	my $decifac = pfget_insist($pf, "$stagename\{decifac}") ;
	$rate /= $decifac ; 
	my $response = pfget_insist($pf, "$stagename\{response}") ;
	if ( $response =~ /^\*\* / ) { 
	    elog_log "bad response in $pfname: $response\n" ; 
	    $bad++ ;
	} else { 
	    my $decifac2 = decimation_factor($response) ; 
	    if ( $decifac2 != $decifac ) { 
		elog_log "different decimation factors: $pfname:$stagename=>$decifac   fir=>$decifac2" ; 
		$bad++ ; 
	    }
	}
    }
    return $rate ;
}

sub validate_stream { 
    my ($pf, $streamname) = @_ ; 
    my $stream = pfget_insist($pf, $streamname) ; 
    my @stages = split(' ', $stream) ;
    my $samprate = shift @stages ; 
    if ( $samprate !~ /^[\d.]+/ ) { 
	elog_log ( "suspicious samprate='$samprate' for stream $streamname" ) ; 
	$bad++ ;
    }
    my $sps = $streamname ; 
    $sps =~ s/[^0-9.]+//ig ;
    if ( $samprate != $sps ) { 
	elog_log ( "suspicious samprate='$samprate' doesn't match stream name $streamname" ) ; 
	$bad++ ;
    }

    my $rate ; 
    foreach my $stagename (@stages) { 
	$rate = validate_stage($pf, $stagename, $rate) ; 
    }
    if ( $samprate != $rate ) { 
	my $pfname = $pf ; 
	$pfname =~ s".*/"" ; 
	elog_log ( "$pfname:$streamname  samprate='$samprate' doesn't match filter stack output=$rate" ) ; 
	$bad++ ;
    }
}


sub validate_datalogger { 
    my ($pf) = @_ ; 
    $bad = 0 ;
    $pf .= ".pf" if $pf !~ /\.pf$/ ; 
    if ( ! -f $pf ) { 
	my $DataloggerDir = pfget($Pf, "DataloggerDir" ) ; 
	if ( $DataloggerDir !~ m"^/" ) { 
	    $DataloggerDir = "$ENV{ANTELOPE}/data/$DataloggerDir" ; 
	}
	$pf = "$DataloggerDir/$pf" ;
    }
    pfget_insist($pf, "category" ) ; 
    pfget_insist($pf, "configuration" ) ; 
    pfget_insist($pf, "description" ) ; 
    pfget_insist($pf, "dfile" ) ; 
    pfget_insist($pf, "gains" ) ; 
    my $streams = pfget_insist($pf, "streams" ) ; 
    foreach my $streamname ( split(' ', $streams) ) { 
	validate_stream ( $pf, $streamname ) ;
    }
    return $bad ;
}

1;
