
package Component ;
use strict ; 
use warnings ; 
use lib "$ENV{ANTELOPE}/data/perl" ;
use Datascope ; 

sub new { 
    my $class = shift ; 
    my $self = {
	axis => "",
	old_axis => "",
	hang => "",
	vang => "",
	nominal_sensitivity => undef,
	sensor_gain => "",
	w_sensor_gain => undef,
	preamp_gain => "",
	preamp_nominal_gain => "",
	preamp_stage => "",
	datalogger_gain => "",
	w_datalogger_gain => undef,
	datalogger_nominal_gain => undef,
	lead => "", 
	by_hand => 0,
	@_} ;
    bless $self, $class ; 
    return $self ;
}

sub by_hand : lvalue { 
    my $self = shift ; 
    $self->{by_hand} ;
}

sub axis : lvalue { 
    my $self = shift ; 
    $self->{axis} ;
}

sub old_axis : lvalue { 
    my $self = shift ; 
    $self->{old_axis} ;
}

sub hang : lvalue { 
    my $self = shift ; 
    $self->{hang} ;
}

sub vang : lvalue { 
    my $self = shift ; 
    $self->{vang} ;
}

sub sensor_gain : lvalue { 
    my $self = shift ; 
    $self->{sensor_gain} ;
}

sub w_sensor_gain : lvalue { 
    my $self = shift ; 
    $self->{w_sensor_gain} ;
}

sub nominal_sensitivity : lvalue { 
    my $self = shift ; 
    $self->{nominal_sensitivity} ;
}

sub preamp_gain : lvalue { 
    my $self = shift ; 
    $self->{preamp_gain} ;
}

sub preamp_nominal_gain : lvalue { 
    my $self = shift ; 
    $self->{preamp_nominal_gain} ;
}

sub preamp_stage : lvalue { 
    my $self = shift ; 
    $self->{preamp_stage} ;
}

sub datalogger_gain : lvalue { 
    my $self = shift ; 
    $self->{datalogger_gain} ;
}

sub w_datalogger_gain : lvalue { 
    my $self = shift ; 
    $self->{w_datalogger_gain} ;
}

sub datalogger_nominal_gain : lvalue { 
    my $self = shift ; 
    $self->{datalogger_nominal_gain} ;
}

sub lead : lvalue { 
    my $self = shift ; 
    $self->{lead} ;
}

sub specified { 
    my ($val) = @_ ; 
    if ( defined $val && $val !~ /^\s*$/ && $val ne "-" ) { 
	return 1 ; 
    } else {
	return 0 ; 
    }
}

sub print {
    my $self = shift ; 
    my $line = sprintf "     axis2 %s %s %s %s %s %s", 
	    $self->old_axis eq $self->axis ? 
	    	$self->axis : $self->old_axis . "->" . $self->axis,  
	    $self->hang, 
	    $self->vang,
	    $self->sensor_gain,
	    $self->datalogger_gain,
	    &specified($self->preamp_gain) ? $self->preamp_gain : "-", 
	    &specified($self->preamp_stage) ? $self->preamp_stage : "",
	    &specified($self->lead) ? $self->lead : "-" ; 
    $line =~ s"( -)+$"" ;
    print "$line\n" ; 
}

sub dump {
    my $self = shift ; 
    print STDERR "nominal sensitivity undefined\n" if ! specified ( $self->nominal_sensitivity ) ; 
    print STDERR "nominal gain undefined\n" if ! specified ( $self->datalogger_nominal_gain ) ; 
    my $line = sprintf "     axis %s hang=%s vang=%s sens=%s (nom=%s) lead=%s preamp=%s stage=%s dlgain=%s (nom=%s) %s\n", 
	    $self->old_axis eq $self->axis ? 
	    	$self->axis : $self->old_axis . "->" . $self->axis,  
	    $self->hang, 
	    $self->vang,
	    $self->sensor_gain,
	    $self->nominal_sensitivity,
	    specified($self->lead) ? $self->lead : "-", 
	    &specified($self->preamp_gain) ? $self->preamp_gain : "-", 
	    &specified($self->preamp_stage) ? $self->preamp_stage : "",
	    $self->datalogger_gain,
	    $self->datalogger_nominal_gain,
	    $self->by_hand ? "(by_hand)" : "(auto)" ;
    print STDERR $line ; 
}


package Channel ; 

sub new { 
    my $class = shift ; 
    my $self = {
	on => 0, 
	component => "",
	chan => "",
	loc => "", 
	id  => "",
	by_hand => 0,
	@_
	} ;
    bless $self, $class ; 
    return $self ;
}

sub on : lvalue { 
    my $self = shift ; 
    $self->{on} ;
}

sub by_hand : lvalue { 
    my $self = shift ; 
    $self->{by_hand} ;
}

sub component : lvalue { 
    my $self = shift ; 
    $self->{component} ;
}

sub chan : lvalue { 
    my $self = shift ; 
    $self->{chan} ;
}

sub loc : lvalue { 
    my $self = shift ; 
    $self->{loc} ;
}

sub id : lvalue { 
    my $self = shift ; 
    $self->{id} ;
}


sub print {
    my $self = shift ; 
    if ( $self->on && $self->by_hand ) { 
	my $line = sprintf "    channel %s  %-3s %-3s %s", 
		$self->component->axis, 
		$self->chan, 
		$self->loc =~ /^\s*$/ ? "-" : $self->loc, 
		$self->id ;
	$line =~ s"( -)+\s*$"" ;
	print "$line\n" ; 
    } elsif ( ! $self->on )  { 
	my $line = sprintf "    channel  %-3s off", $self->component->axis ;  
	print "$line\n" ; 
    }
}

sub dump {
    my $self = shift ; 
    printf STDERR "         channel  %-3s %-3s %-3s %s (%s)\n", 
	    $self->on ? "on" : "off", $self->chan, $self->loc, $self->id, 
	    $self->by_hand ? "by_hand" : "auto" ;
    printf STDERR "          "; $self->component->dump ; 
}


package SampleRate ; 

sub new { 
    my $class = shift ; 
    my $self = {
	choice => "",
	samprate => "",
	name => "",
	channels => [],
	stages => [],
	@_} ;
    bless $self, $class ; 
    return $self ;
}


sub name : lvalue { 
    my $self = shift ; 
    $self->{name} ;
}

sub choice : lvalue { 
    my $self = shift ; 
    $self->{choice} ;
}

sub samprate : lvalue { 
    my $self = shift ; 
    $self->{samprate} ;
}

sub channels : lvalue { 
    my $self = shift ; 
    $self->{channels} ;
}

sub stages : lvalue { 
    my $self = shift ; 
    $self->{stages} ;
}

sub print {
    my $self = shift ; 
    printf "   samplerate %s\n", $self->choice ;
    my $channels = $self->channels ;
    my $by_hand = 1 ;
    foreach (@$channels) { 
	$by_hand += $_->by_hand ; 
    }
    if ( $by_hand ) { 
	foreach (@$channels) { 
	    $_->print() ; 
	}
    }
}

sub dump {
    my $self = shift ; 
    printf STDERR "   samplerate %s # %s\n", $self->choice, $self->samprate ;
    my $stages = $self->stages ; 
    print STDERR "     # stages: @$stages\n" ; 
    my $channels = $self->channels ;
    foreach (@$channels) { 
	$_->dump() ; 
    }
}


package Sensor ; 

sub new { 
    my $class = shift ; 
    my $self = {
	index => 1,
	pf => undef,
	dfile => undef,
	snchoice => undef,
	sndesc => undef,
	sensor_serialnumber => undef,
	edepth => undef,
	rsptype => undef,
	band => undef,
	sngnom => 1, 
	sensor_loc => "",
	names => [],
	components => [],
	selected_samplerate => undef, 
	samplerates => [],
	units => "",
	widget => {},
	w => undef, 
    	@_
	} ;
    bless $self, $class ; 
    return $self ;
}

sub snchoice : lvalue { 
    my $self = shift ; 
    $self->{snchoice} ;
}

sub sngnom : lvalue { 
    my $self = shift ; 
    $self->{sngnom} ;
}

sub w : lvalue { 
    my $self = shift ; 
    $self->{w} ;
}

sub units : lvalue { 
    my $self = shift ; 
    $self->{units} ;
}

sub widget : lvalue { 
    my $self = shift ; 
    $self->{widget} ;
}

sub pf : lvalue { 
    my $self = shift ; 
    $self->{pf} ;
}

sub index : lvalue { 
    my $self = shift ; 
    $self->{index} ;
}

sub sndesc : lvalue { 
    my $self = shift ; 
    $self->{sndesc} ;
}

sub dfile : lvalue { 
    my $self = shift ; 
    $self->{dfile} ;
}

sub sensor_serialnumber : lvalue { 
    my $self = shift ; 
    $self->{sensor_serialnumber} ;
}

sub sensor_loc : lvalue { 
    my $self = shift ; 
    $self->{sensor_loc} ;
}

sub rsptype : lvalue { 
    my $self = shift ; 
    $self->{rsptype} ;
}

sub band : lvalue { 
    my $self = shift ; 
    $self->{band} ;
}

sub edepth : lvalue { 
    my $self = shift ; 
    $self->{edepth} ;
}

sub names : lvalue { 
    my $self = shift ; 
    $self->{names} ;
}

sub components : lvalue { 
    my $self = shift ; 
    $self->{components} ;
}

sub selected_samplerate : lvalue { 
    my $self = shift ; 
    $self->{selected_samplerate} ;
}

sub samplerates : lvalue { 
    my $self = shift ; 
    $self->{samplerates} ;
}

sub p { 
    my ($a) = @_ ;
    if ( ! defined $a ) { 
	$a = "" ; 
    } 
    return $a ; 
}

sub print {
    my $self = shift ; 
    printf "\nsensor %s %s %s %s # %s\n", p($self->snchoice), p($self->edepth), p($self->sensor_serialnumber), p($self->sensor_loc), 
	    p($self->sndesc) ;
    my $ref = $self->components() ;
    foreach  ( @$ref ) { 
	$_->print() if $_->by_hand ;
    }
    $ref = $self->samplerates() ;
    foreach  ( @$ref ) { 
	if ($_->samprate ne "" && $_->samprate > 0) {
	    $_->print() ;
	}
    }
}

sub dump {
    my $self = shift ; 
    printf STDERR "\nsensor %s\n", $self->snchoice ;
    printf STDERR "# %s\n", $self->sndesc ;
    printf STDERR "   # pf %s\n", $self->pf ;
    printf STDERR "   # rsptype %s\n", $self->rsptype ;
    printf STDERR "   # band %s\n", $self->band ;
    printf STDERR "   # dfile %s\n", $self->dfile ;
    printf STDERR "   sensor_serialnumber %s\n", $self->sensor_serialnumber  ;
    printf STDERR "   edepth %s\n", $self->edepth  ;
    printf STDERR "   sensor_loc %s\n", $self->sensor_loc  ;
    my $ref = $self->components() ;
    foreach  ( @$ref ) { 
	$_->dump() ; 
    }
    $ref = $self->samplerates() ;
    my $i = 0 ; 
    printf STDERR "%d samplerates\n", scalar(@$ref) ; 
    foreach  ( @$ref ) { 
	printf STDERR "      Samplerate #$i :" ;
	$_->dump() ;
	$i++ ;
    }
    if ( defined $self->selected_samplerate 
      && $self->selected_samplerate->samprate ne "" 
      && $self->selected_samplerate->samprate > 0 ) {
	printf STDERR "    selected samplerate: \n", 
		$self->selected_samplerate->samprate ; 
    }
}

package Datalogger ;

sub new { 
    my $class = shift ; 
    my $self = {
	dlchoice => undef,  # used by browseentry
	dlchosen => undef,  # filled in by program
	dldesc => undef,
	datalogger_serialnumber => undef,
        dfile => undef,
	datalogger_id => "",
	pf => undef,
	possible_samprates => [],
    	@_
	} ;
    bless $self, $class ; 
    return $self ;
}

sub possible_samprates : lvalue { 
    my $self = shift ; 
    $self->{possible_samprates} ;
}

sub dlchosen : lvalue { 
    my $self = shift ; 
    $self->{dlchosen} ;
}

sub dlchoice : lvalue { 
    my $self = shift ; 
    $self->{dlchoice} ;
}

sub dldesc : lvalue { 
    my $self = shift ; 
    $self->{dldesc} ;
}

sub datalogger_serialnumber : lvalue { 
    my $self = shift ; 
    $self->{datalogger_serialnumber} ;
}

sub datalogger_id : lvalue { 
    my $self = shift ; 
    $self->{datalogger_id} ;
}

sub dfile : lvalue { 
    my $self = shift ; 
    $self->{dfile} ;
}

sub pf : lvalue { 
    my $self = shift ; 
    $self->{pf} ;
}

sub print {
    my $self = shift ; 
    printf "\ndatalogger %s %s %s # %s\n", 
	$self->dlchoice,  
	$self->datalogger_serialnumber,
    	$self->datalogger_id, 
	$self->dldesc ;
    print "# bad datalogger_dlchoice\n" if ( ! defined $self->dlchoice ) ;
    print "# bad datalogger_serialnumber\n" if ( ! defined $self->datalogger_serialnumber ) ;
    print "# bad datalogger_id\n" if ( ! defined $self->datalogger_id ) ;
    print "# bad datalogger_dldesc\n" if ( ! defined $self->dldesc ) ;
}

sub dump {
    my $self = shift ; 
    printf STDERR "\ndatalogger %s %s %s\n", $self->dlchoice,  $self->datalogger_serialnumber, $self->datalogger_id ;
    printf STDERR "# %s\n", $self->dldesc ;
}

package Station ;

sub new { 
    my $class = shift ; 
    my $self = {
	sta => "",
	lat => "",
	lon => "",
	elev => "",
	staname => "",
    	@_
	} ;
    bless $self, $class ; 
    return $self ;
}

sub sta : lvalue { 
    my $self = shift ; 
    $self->{sta} ;
}

sub lat : lvalue { 
    my $self = shift ; 
    $self->{lat} ;
}

sub lon : lvalue { 
    my $self = shift ; 
    $self->{lon} ;
}

sub elev : lvalue { 
    my $self = shift ; 
    $self->{elev} ;
}

sub staname : lvalue { 
    my $self = shift ; 
    $self->{staname} ;
}

sub print {
    my $self = shift ; 
    printf "sta %s %s %s %s %s\n", 
	$self->sta, $self->lat, $self->lon, $self->elev, $self->staname ;
}

sub dump {
    my $self = shift ; 
    printf STDERR "sta %s %s %s %s %s\n", 
	$self->sta, $self->lat, $self->lon, $self->elev, $self->staname ;
}

package Configuration ;

sub new { 
    my $class = shift ; 
    my $self = {
	id => "",
	comment => "", 
	time => undef,
	net => undef,
	netname => undef,
	station => undef,
	datalogger => undef,
	selected_sensor => undef,
	index => 1,
	sensors => [],
	possible_sensors => [],
	possible_dataloggers => [], 
	templates => [],
	template => undef,
	TemplateDir => undef,
	SensorDir => undef,
	DataloggerDir => undef,
	SerialNumberDir => undef,
	PreampDir => undef,
	preamps => [],
    	@_
    } ;
    bless $self, $class ; 
    return $self ;
}

sub clear { 
    my $self = shift ; 
    $self->id = "" ;
    $self->comment = "" ;
    $self->time = undef ;
    $self->net = undef ;
    $self->netname = undef ;
    $self->station = undef ;
    $self->datalogger = undef ;
    $self->selected_sensor = undef ;
    $self->index = 1 ;
    $self->sensors = [] ;
    $self->template = undef ;
}

sub id : lvalue { 
    my $self = shift ; 
    $self->{id} ;
}

sub comment : lvalue { 
    my $self = shift ; 
    $self->{comment} ;
}

sub SensorDir : lvalue { 
    my $self = shift ; 
    $self->{SensorDir} ;
}

sub DataloggerDir : lvalue { 
    my $self = shift ; 
    $self->{DataloggerDir} ;
}

sub SerialNumberDir : lvalue {
    my $self = shift ; 
    $self->{SerialNumberDir} ;
}

sub TemplateDir : lvalue { 
    my $self = shift ; 
    $self->{TemplateDir} ;
}

sub PreampDir : lvalue { 
    my $self = shift ; 
    $self->{PreampDir} ;
}

sub possible_sensors : lvalue { 
    my $self = shift ; 
    $self->{possible_sensors} ;
}

sub possible_dataloggers : lvalue { 
    my $self = shift ; 
    $self->{possible_dataloggers} ;
}

sub templates : lvalue { 
    my $self = shift ; 
    $self->{templates} ;
}

sub template : lvalue { 
    my $self = shift ; 
    $self->{template} ;
}

sub preamps : lvalue { 
    my $self = shift ; 
    $self->{preamps} ;
}

sub net : lvalue { 
    my $self = shift ; 
    $self->{net} ;
}

sub netname : lvalue { 
    my $self = shift ; 
    $self->{netname} ;
}

sub index : lvalue { 
    my $self = shift ; 
    $self->{index} ;
}

sub time : lvalue { 
    my $self = shift ; 
    $self->{time} ;
}

sub selected_sensor : lvalue { 
    my $self = shift ; 
    $self->{selected_sensor} ;
}

sub station : lvalue { 
    my $self = shift ; 
    $self->{station} ;
}

sub datalogger : lvalue { 
    my $self = shift ; 
    $self->{datalogger} ;
}

sub sensors : lvalue { 
    my $self = shift ; 
    $self->{sensors} ;
}


use Datascope ;

sub specified { 
    my ($val) = @_ ; 
    if ( defined $val && $val !~ /^\s*$/ && $val ne "-" ) { 
	return 1 ; 
    } else {
	return 0 ; 
    }
}

sub print {
    my $self = shift ; 
    printf "\n# %s\n", $self->id ;
    printf "# %s\n", $self->comment if specified($self->comment) ;
    if ( $self->time ne "" ) { 
	my $time = is_epoch_string($self->time) ; 
	printf "time %s\n", &strtime($time) if defined $time ; 
    }
    printf "\nnet %s %s\n", $self->net, $self->netname if defined $self->net ; 
    $self->station->print() if defined $self->station ; 
    $self->datalogger->print() if defined $self->datalogger ; 
    my $ref = $self->sensors() ;
    foreach (@$ref) {
	if (defined $_->dfile) {  
	    $_->print();
	}
    }
}

sub dump {
    my $self = shift ; 
    printf STDERR "\n# %s\n", $self->id ;
    printf STDERR "\n# %s\n", $self->comment if specified($self->comment) ;
    if ( $self->time ne "" ) { 
	my $time = is_epoch_string($self->time) ; 
	printf STDERR "time %s\n", &strtime($time) if defined $time ; 
    }
    printf STDERR "\nnet %s %s\n", $self->net, $self->netname ; 
    $self->station->dump() ; 
    $self->datalogger->dump() ; 
    printf STDERR "sensor index #%s\n", $self->index ;
    printf STDERR "selected_sensor %s\n", $self->selected_sensor->snchoice ;
    my $ref = $self->sensors() ;
    my $i = 0 ; 
    foreach (@$ref) {
	print STDERR "sensor #$i\n" ; 
	$_->dump() ;
	$i++ ;
    }
    printf STDERR "TemplateDir '%s'\n", $self->TemplateDir ; 
    printf STDERR "SensorDir '%s'\n", $self->SensorDir ; 
    printf STDERR "DataloggerDir '%s'\n", $self->DataloggerDir ; 
    printf STDERR "PreampDir '%s'\n\n", $self->PreampDir ; 
    printf STDERR "The End\n\n" ; 
}
    
1;
